@@ -2,7 +2,6 @@ import re
 import traceback
 from os.path import join
 from pathlib import Path
-from pprint import pprint
 from sys import stdout
 from typing import Union
 import json
@@ -26,13 +25,17 @@ class Evaluator():
         for match in re.finditer(r"\@([^\(]+)\(", expr):
             if match:
                 func = re.sub(r'\@', '', match.group(1))
+                # print('--func--', func)
+                # print('--expr--', expr)
                 arg_match = re.search(
-                    f"{func}\\((?<!')([^\\)]+)(?!')\\)", expr, flags=re.MULTILINE)
+                    f"{func}\\((?<!')(.*?)\\)(?!')", expr, flags=re.MULTILINE)
                 args = {}
                 if arg_match:
                     arg_str = arg_match.group(1).strip()
+                    # print('---')
+                    # print('--arg_str--', arg_str)
+                    # print('---')
                     args = eval(f"dict({arg_str})")
-
                 if func in self.functions:
                     result = self.functions[func](**args)
                     expr = re.sub(
@@ -41,13 +44,23 @@ class Evaluator():
                     log.warning(f"Function '{func}' is not defined.")
         return expr
 
+    # @staticmethod
+    # def __clean_var_value(value):
+    #     # print('--value--', value)
+    #     if isinstance(value, str):
+    #         return re.sub(r'\+', r' ', value)
+    #     else:
+    #         return value
+
     def __replace_vars(self, expr: str, context: dict, informative=False):
         for match in re.finditer(self.RE_VAR, expr):
             if match:
                 var = re.sub(r'\$', '', match.group(0))
                 if var in context['vars']:
+                    # print('--var--', var)
                     expr = re.sub(
                         f"\\${var}", f"{context['vars'][var]}{' (' + var + ')' if informative else ''}", expr)
+                    # print('--expr--', expr)
                 elif var in context['consts']:
                     expr = re.sub(
                         f"\\${var}", f"{context['consts'][var]}{' (' + var + ')' if informative else ''}", expr)
@@ -80,6 +93,7 @@ class Evaluator():
     def __eval_and_or(self, condition: Union[dict, list, str], context: dict):
         evaluation = ''
         if isinstance(condition, list):
+            # print('--condition--', condition)
             for cond in condition:
                 evaluation += f"{self.__eval_and_or(cond, context)}"
         elif isinstance(condition, dict):
@@ -92,6 +106,8 @@ class Evaluator():
             else:
                 op = None
             if op:
+                if condition[op] is None:
+                    raise Exception(f"'{op}'' must not be empty.", condition)
                 for idx, cond in enumerate(condition[op]):
                     if idx == 0:
                         evaluation += '('
@@ -100,11 +116,12 @@ class Evaluator():
                         evaluation += ')'
                     else:
                         evaluation += f" {op} "
-                # print(evaluation)
-                # print(self.__replace_vars(evaluation, context))
-                # print(self.__replace_functions(evaluation, context))
+                # # print(evaluation)
+                # # print(self.__replace_vars(evaluation, context))
+                # # print(self.__replace_functions(evaluation, context))
             else:
-                raise(Exception("Only 'and' and 'or' are allowed operators.", condition))
+                raise Exception(
+                    "Only 'and' and 'or' are allowed operators.", condition)
         else:
             return f"( {condition} )"
         return self.__eval_condition(evaluation, context)
@@ -126,6 +143,7 @@ class Evaluator():
             finally:
                 return result
         elif isinstance(condition, list):
+            # print('--initial condition--', condition)
             return self.__eval_and_or(condition, context)
 
     def __evaluate_variables(self, _vars: dict, obj: dict, context: dict):
@@ -178,16 +196,15 @@ class Evaluator():
                     if isinstance(result[field], dict):
                         result[field].update(
                             {'evaluation': self.__get_evaluation(op['fields'][field], context)})
-
             # if its a for operation
             if 'for' in op:
                 block = op['for']
                 if 'field' not in block:
-                    raise(Exception("'field' is required in 'for'.", block))
+                    raise Exception("'field' is required in 'for'.", block)
                 if 'as' not in block:
-                    raise(Exception("'as' is required in 'for'.", block))
+                    raise Exception("'as' is required in 'for'.", block)
                 if 'evaluate' not in block:
-                    raise(Exception("'evaluate' is required in 'for'.", block))
+                    raise Exception("'evaluate' is required in 'for'.", block)
                 for idx, item in enumerate(dotpath_get_value(obj, block['field'])):
                     context['vars'].update({
                         '__field__': block['field'],
@@ -223,7 +240,7 @@ class Evaluator():
                 if key == field:
                     found.update(item[key])
                 else:
-                    # print('next', item[key])
+                    # # print('next', item[key])
                     self.__get_field(item[key], field, found)
         return found
 
